( cd usr/share/doc/HTML/en/k3b ; rm -rf common )
( cd usr/share/doc/HTML/en/k3b ; ln -sf /usr/share/doc/HTML/en/common common )
( cd usr/lib ; rm -rf libk3bdevice.so )
( cd usr/lib ; ln -sf libk3bdevice.so.5.0.0 libk3bdevice.so )
( cd usr/lib ; rm -rf libk3b.so )
( cd usr/lib ; ln -sf libk3b.so.3.0.0 libk3b.so )
( cd usr/lib ; rm -rf libk3bdevice.so.5 )
( cd usr/lib ; ln -sf libk3bdevice.so.5.0.0 libk3bdevice.so.5 )
( cd usr/lib ; rm -rf libk3b.so.3 )
( cd usr/lib ; ln -sf libk3b.so.3.0.0 libk3b.so.3 )
