<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="k3b" version="8">
<MenuBar>
  <Menu name="project"><text>&amp;Project</text>
  	<Action name="project_add_files" />
	<Action name="project_clear_project" />
  </Menu>

  <Menu name="tools"><text>&amp;Tools</text>
	<Action name="tools_copy_cd" />
  	<Action name="tools_copy_dvd" />
	<Separator />
	<Action name="tools_blank_cdrw" />
	<Action name="tools_format_dvd" />
	<Separator />
	<Action name="tools_write_cd_image" />
	<Action name="tools_write_dvd_iso" />
	<Separator />
	<Action name="tools_cdda_rip" />
	<Action name="tools_videodvd_rip" />
	<Action name="tools_videocd_rip" />
  </Menu>

  <Menu name="device"><text>&amp;Device</text>
	<Action name ="device_diskinfo" />
	<Separator />
	<Action name ="device_unmount" />
	<Action name ="device_mount" />
	<Separator />
	<Action name ="device_eject" />
	<Action name ="device_load" />
	<Separator />
	<Action name ="device_set_read_speed" />
  </Menu>

  <Menu name="settings">
  	<Action name="view_show_project_view" append="show_merge" />
  	<Action name="view_dir_tree" append="show_merge" />
  	<Action name="view_contents" append="show_merge" />
  	<Action name="view_audio_player" append="show_merge" />
  	<Action name="view_document_header" append="show_merge" />
	<Action name="settings_k3bsetup" append="configure_merge" />
  </Menu>

  <Menu name="help"><text>&amp;Help</text>
	<Action name="help_check_system" />	
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
	<Action name="file_new" />
	<Action name="file_open" />
	<Action name="file_save" />
</ToolBar>

<ToolBar name="toolsToolBar"><text>Tools</text>
	<Action name="tools_copy_cd" />
	<Action name="tools_copy_dvd" />
	<Action name="tools_blank_cdrw" />
	<Action name="tools_format_dvd" />
</ToolBar>
	
<ToolBar name="quickDirSelector" fullWidth="true"><text>Quick Dir Selector</text>
	<Action name="quick_dir_selector" />
	<Action name="go_url" />
</ToolBar>

<State name="state_project_active">
	<enable>
		<Action name="file_save" />
		<Action name="file_save_as" />
		<Action name="file_close" />
		<Action name="file_close_all" />
	  	<Action name="project_add_files" />		
		<Action name="project_clear_project" />
	</enable>
</State>
</kpartgui>
